@@ -1,12 +1,12 @@
 //! Combinators applying their child parser multiple times
 
+use crate::combinator::trace;
 use crate::error::ErrMode;
 use crate::error::ErrorKind;
 use crate::error::ParserError;
 use crate::stream::Accumulate;
 use crate::stream::Range;
 use crate::stream::Stream;
-use crate::trace::trace;
 use crate::PResult;
 use crate::Parser;
 
@@ -201,6 +201,73 @@ where
     }
 }
 
+fn repeat_n_<I, O, C, E, F>(count: usize, f: &mut F, i: &mut I) -> PResult<C, E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    F: Parser<I, O, E>,
+    E: ParserError<I>,
+{
+    let mut res = C::initial(Some(count));
+
+    for _ in 0..count {
+        match f.parse_next(i) {
+            Ok(o) => {
+                res.accumulate(o);
+            }
+            Err(e) => {
+                return Err(e.append(i, ErrorKind::Many));
+            }
+        }
+    }
+
+    Ok(res)
+}
+
+fn repeat_m_n_<I, O, C, E, F>(min: usize, max: usize, parse: &mut F, input: &mut I) -> PResult<C, E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    F: Parser<I, O, E>,
+    E: ParserError<I>,
+{
+    if min > max {
+        return Err(ErrMode::Cut(E::from_error_kind(input, ErrorKind::Many)));
+    }
+
+    let mut res = C::initial(Some(min));
+    for count in 0..max {
+        let start = input.checkpoint();
+        let len = input.eof_offset();
+        match parse.parse_next(input) {
+            Ok(value) => {
+                // infinite loop check: the parser must always consume
+                if input.eof_offset() == len {
+                    return Err(ErrMode::assert(
+                        input,
+                        "`repeat` parsers must always consume",
+                    ));
+                }
+
+                res.accumulate(value);
+            }
+            Err(ErrMode::Backtrack(e)) => {
+                if count < min {
+                    return Err(ErrMode::Backtrack(e.append(input, ErrorKind::Many)));
+                } else {
+                    input.reset(start);
+                    return Ok(res);
+                }
+            }
+            Err(e) => {
+                return Err(e);
+            }
+        }
+    }
+
+    Ok(res)
+}
+
 /// [`Accumulate`] the output of parser `f` into a container, like `Vec`, until the parser `g`
 /// produces a result.
 ///
@@ -216,11 +283,11 @@ where
 /// # #[cfg(feature = "std")] {
 /// # use winnow::{error::ErrMode, error::{InputError, ErrorKind}, error::Needed};
 /// # use winnow::prelude::*;
-/// use winnow::combinator::repeat_till0;
+/// use winnow::combinator::repeat_till;
 /// use winnow::token::tag;
 ///
 /// fn parser(s: &str) -> IResult<&str, (Vec<&str>, &str)> {
-///   repeat_till0("abc", "end").parse_peek(s)
+///   repeat_till(0.., "abc", "end").parse_peek(s)
 /// };
 ///
 /// assert_eq!(parser("abcabcend"), Ok(("", (vec!["abc", "abc"], "end"))));
@@ -231,7 +298,11 @@ where
 /// # }
 /// ```
 #[doc(alias = "many_till0")]
-pub fn repeat_till0<I, O, C, P, E, F, G>(mut f: F, mut g: G) -> impl Parser<I, (C, P), E>
+pub fn repeat_till<I, O, C, P, E, F, G>(
+    range: impl Into<Range>,
+    mut f: F,
+    mut g: G,
+) -> impl Parser<I, (C, P), E>
 where
     I: Stream,
     C: Accumulate<O>,
@@ -239,32 +310,255 @@ where
     G: Parser<I, P, E>,
     E: ParserError<I>,
 {
-    trace("repeat_till0", move |i: &mut I| {
-        let mut res = C::initial(None);
-        loop {
-            let start = i.checkpoint();
-            let len = i.eof_offset();
-            match g.parse_next(i) {
-                Ok(o) => return Ok((res, o)),
-                Err(ErrMode::Backtrack(_)) => {
-                    i.reset(start);
-                    match f.parse_next(i) {
-                        Err(e) => return Err(e.append(i, ErrorKind::Many)),
-                        Ok(o) => {
-                            // infinite loop check: the parser must always consume
-                            if i.eof_offset() == len {
-                                return Err(ErrMode::assert(
-                                    i,
-                                    "`repeat` parsers must always consume",
-                                ));
-                            }
+    let Range {
+        start_inclusive,
+        end_inclusive,
+    } = range.into();
+    trace("repeat_till", move |i: &mut I| {
+        match (start_inclusive, end_inclusive) {
+            (0, None) => repeat_till0_(&mut f, &mut g, i),
+            (start, end) => repeat_till_m_n_(start, end.unwrap_or(usize::MAX), &mut f, &mut g, i),
+        }
+    })
+}
 
-                            res.accumulate(o);
+/// Deprecated, replaced with [`repeat_till`]
+#[deprecated(since = "0.5.35", note = "Replaced with `repeat_till`")]
+#[inline(always)]
+pub fn repeat_till0<I, O, C, P, E, F, G>(f: F, g: G) -> impl Parser<I, (C, P), E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    F: Parser<I, O, E>,
+    G: Parser<I, P, E>,
+    E: ParserError<I>,
+{
+    repeat_till(0.., f, g)
+}
+
+fn repeat_till0_<I, O, C, P, E, F, G>(f: &mut F, g: &mut G, i: &mut I) -> PResult<(C, P), E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    F: Parser<I, O, E>,
+    G: Parser<I, P, E>,
+    E: ParserError<I>,
+{
+    let mut res = C::initial(None);
+    loop {
+        let start = i.checkpoint();
+        let len = i.eof_offset();
+        match g.parse_next(i) {
+            Ok(o) => return Ok((res, o)),
+            Err(ErrMode::Backtrack(_)) => {
+                i.reset(start);
+                match f.parse_next(i) {
+                    Err(e) => return Err(e.append(i, ErrorKind::Many)),
+                    Ok(o) => {
+                        // infinite loop check: the parser must always consume
+                        if i.eof_offset() == len {
+                            return Err(ErrMode::assert(i, "`repeat` parsers must always consume"));
                         }
+
+                        res.accumulate(o);
                     }
                 }
-                Err(e) => return Err(e),
             }
+            Err(e) => return Err(e),
+        }
+    }
+}
+
+fn repeat_till_m_n_<I, O, C, P, E, F, G>(
+    min: usize,
+    max: usize,
+    f: &mut F,
+    g: &mut G,
+    i: &mut I,
+) -> PResult<(C, P), E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    F: Parser<I, O, E>,
+    G: Parser<I, P, E>,
+    E: ParserError<I>,
+{
+    if min > max {
+        return Err(ErrMode::Cut(E::from_error_kind(i, ErrorKind::Many)));
+    }
+
+    let mut res = C::initial(Some(min));
+    for _ in 0..min {
+        match f.parse_next(i) {
+            Ok(o) => {
+                res.accumulate(o);
+            }
+            Err(e) => {
+                return Err(e.append(i, ErrorKind::Many));
+            }
+        }
+    }
+    for count in min..=max {
+        let start = i.checkpoint();
+        let len = i.eof_offset();
+        match g.parse_next(i) {
+            Ok(o) => return Ok((res, o)),
+            Err(ErrMode::Backtrack(err)) => {
+                if count == max {
+                    return Err(ErrMode::Backtrack(err));
+                }
+                i.reset(start);
+                match f.parse_next(i) {
+                    Err(e) => {
+                        return Err(e.append(i, ErrorKind::Many));
+                    }
+                    Ok(o) => {
+                        // infinite loop check: the parser must always consume
+                        if i.eof_offset() == len {
+                            return Err(ErrMode::assert(i, "`repeat` parsers must always consume"));
+                        }
+
+                        res.accumulate(o);
+                    }
+                }
+            }
+            Err(e) => return Err(e),
+        }
+    }
+    unreachable!()
+}
+
+/// [`Accumulate`] the output of a parser, interleaved with `sep`
+///
+/// This stops when either parser returns [`ErrMode::Backtrack`]. To instead chain an error up, see
+/// [`cut_err`][crate::combinator::cut_err].
+///
+/// # Arguments
+/// * `range` The minimum and maximum number of iterations.
+/// * `parser` The parser that parses the elements of the list.
+/// * `sep` The parser that parses the separator between list elements.
+///
+/// **Warning:** If the separator parser accepts empty inputs
+/// (like `alpha0` or `digit0`), `separated` will return an error,
+/// to prevent going into an infinite loop.
+///
+/// # Example
+///
+/// Zero or more repetitions:
+/// ```rust
+/// # #[cfg(feature = "std")] {
+/// # use winnow::{error::ErrMode, error::ErrorKind, error::Needed};
+/// # use winnow::prelude::*;
+/// use winnow::combinator::separated;
+/// use winnow::token::tag;
+///
+/// fn parser(s: &str) -> IResult<&str, Vec<&str>> {
+///   separated(0.., "abc", "|").parse_peek(s)
+/// }
+///
+/// assert_eq!(parser("abc|abc|abc"), Ok(("", vec!["abc", "abc", "abc"])));
+/// assert_eq!(parser("abc123abc"), Ok(("123abc", vec!["abc"])));
+/// assert_eq!(parser("abc|def"), Ok(("|def", vec!["abc"])));
+/// assert_eq!(parser(""), Ok(("", vec![])));
+/// assert_eq!(parser("def|abc"), Ok(("def|abc", vec![])));
+/// # }
+/// ```
+///
+/// One or more repetitions:
+/// ```rust
+/// # #[cfg(feature = "std")] {
+/// # use winnow::{error::ErrMode, error::{InputError, ErrorKind}, error::Needed};
+/// # use winnow::prelude::*;
+/// use winnow::combinator::separated;
+/// use winnow::token::tag;
+///
+/// fn parser(s: &str) -> IResult<&str, Vec<&str>> {
+///   separated(1.., "abc", "|").parse_peek(s)
+/// }
+///
+/// assert_eq!(parser("abc|abc|abc"), Ok(("", vec!["abc", "abc", "abc"])));
+/// assert_eq!(parser("abc123abc"), Ok(("123abc", vec!["abc"])));
+/// assert_eq!(parser("abc|def"), Ok(("|def", vec!["abc"])));
+/// assert_eq!(parser(""), Err(ErrMode::Backtrack(InputError::new("", ErrorKind::Tag))));
+/// assert_eq!(parser("def|abc"), Err(ErrMode::Backtrack(InputError::new("def|abc", ErrorKind::Tag))));
+/// # }
+/// ```
+///
+/// Fixed number of repetitions:
+/// ```rust
+/// # #[cfg(feature = "std")] {
+/// # use winnow::{error::ErrMode, error::{InputError, ErrorKind}, error::Needed};
+/// # use winnow::prelude::*;
+/// use winnow::combinator::separated;
+/// use winnow::token::tag;
+///
+/// fn parser(s: &str) -> IResult<&str, Vec<&str>> {
+///   separated(2, "abc", "|").parse_peek(s)
+/// }
+///
+/// assert_eq!(parser("abc|abc|abc"), Ok(("|abc", vec!["abc", "abc"])));
+/// assert_eq!(parser("abc123abc"), Err(ErrMode::Backtrack(InputError::new("123abc", ErrorKind::Tag))));
+/// assert_eq!(parser("abc|def"), Err(ErrMode::Backtrack(InputError::new("def", ErrorKind::Tag))));
+/// assert_eq!(parser(""), Err(ErrMode::Backtrack(InputError::new("", ErrorKind::Tag))));
+/// assert_eq!(parser("def|abc"), Err(ErrMode::Backtrack(InputError::new("def|abc", ErrorKind::Tag))));
+/// # }
+/// ```
+///
+/// Arbitrary repetitions:
+/// ```rust
+/// # #[cfg(feature = "std")] {
+/// # use winnow::{error::ErrMode, error::{InputError, ErrorKind}, error::Needed};
+/// # use winnow::prelude::*;
+/// use winnow::combinator::separated;
+/// use winnow::token::tag;
+///
+/// fn parser(s: &str) -> IResult<&str, Vec<&str>> {
+///   separated(0..=2, "abc", "|").parse_peek(s)
+/// }
+///
+/// assert_eq!(parser("abc|abc|abc"), Ok(("|abc", vec!["abc", "abc"])));
+/// assert_eq!(parser("abc123abc"), Ok(("123abc", vec!["abc"])));
+/// assert_eq!(parser("abc|def"), Ok(("|def", vec!["abc"])));
+/// assert_eq!(parser(""), Ok(("", vec![])));
+/// assert_eq!(parser("def|abc"), Ok(("def|abc", vec![])));
+/// # }
+/// ```
+#[doc(alias = "sep_by")]
+#[doc(alias = "sep_by1")]
+#[doc(alias = "separated_list0")]
+#[doc(alias = "separated_list1")]
+#[doc(alias = "separated_m_n")]
+#[inline(always)]
+pub fn separated<I, O, C, O2, E, P, S>(
+    range: impl Into<Range>,
+    mut parser: P,
+    mut separator: S,
+) -> impl Parser<I, C, E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    P: Parser<I, O, E>,
+    S: Parser<I, O2, E>,
+    E: ParserError<I>,
+{
+    let Range {
+        start_inclusive,
+        end_inclusive,
+    } = range.into();
+    trace("separated", move |input: &mut I| {
+        match (start_inclusive, end_inclusive) {
+            (0, None) => separated0_(&mut parser, &mut separator, input),
+            (1, None) => separated1_(&mut parser, &mut separator, input),
+            (start, end) if Some(start) == end => {
+                separated_n_(start, &mut parser, &mut separator, input)
+            }
+            (start, end) => separated_m_n_(
+                start,
+                end.unwrap_or(usize::MAX),
+                &mut parser,
+                &mut separator,
+                input,
+            ),
         }
     })
 }
@@ -300,6 +594,7 @@ where
 /// ```
 #[doc(alias = "sep_by")]
 #[doc(alias = "separated_list0")]
+#[deprecated(since = "0.5.19", note = "Replaced with `combinator::separated`")]
 pub fn separated0<I, O, C, O2, E, P, S>(mut parser: P, mut sep: S) -> impl Parser<I, C, E>
 where
     I: Stream,
@@ -309,49 +604,67 @@ where
     E: ParserError<I>,
 {
     trace("separated0", move |i: &mut I| {
-        let mut res = C::initial(None);
+        separated0_(&mut parser, &mut sep, i)
+    })
+}
 
-        let start = i.checkpoint();
-        match parser.parse_next(i) {
+fn separated0_<I, O, C, O2, E, P, S>(
+    parser: &mut P,
+    separator: &mut S,
+    input: &mut I,
+) -> PResult<C, E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    P: Parser<I, O, E>,
+    S: Parser<I, O2, E>,
+    E: ParserError<I>,
+{
+    let mut acc = C::initial(None);
+
+    let start = input.checkpoint();
+    match parser.parse_next(input) {
+        Err(ErrMode::Backtrack(_)) => {
+            input.reset(start);
+            return Ok(acc);
+        }
+        Err(e) => return Err(e),
+        Ok(o) => {
+            acc.accumulate(o);
+        }
+    }
+
+    loop {
+        let start = input.checkpoint();
+        let len = input.eof_offset();
+        match separator.parse_next(input) {
             Err(ErrMode::Backtrack(_)) => {
-                i.reset(start);
-                return Ok(res);
+                input.reset(start);
+                return Ok(acc);
             }
             Err(e) => return Err(e),
-            Ok(o) => {
-                res.accumulate(o);
-            }
-        }
-
-        loop {
-            let start = i.checkpoint();
-            let len = i.eof_offset();
-            match sep.parse_next(i) {
-                Err(ErrMode::Backtrack(_)) => {
-                    i.reset(start);
-                    return Ok(res);
+            Ok(_) => {
+                // infinite loop check
+                if input.eof_offset() == len {
+                    return Err(ErrMode::assert(
+                        input,
+                        "`separated` separator parser must always consume",
+                    ));
                 }
-                Err(e) => return Err(e),
-                Ok(_) => {
-                    // infinite loop check: the parser must always consume
-                    if i.eof_offset() == len {
-                        return Err(ErrMode::assert(i, "sep parsers must always consume"));
-                    }
 
-                    match parser.parse_next(i) {
-                        Err(ErrMode::Backtrack(_)) => {
-                            i.reset(start);
-                            return Ok(res);
-                        }
-                        Err(e) => return Err(e),
-                        Ok(o) => {
-                            res.accumulate(o);
-                        }
+                match parser.parse_next(input) {
+                    Err(ErrMode::Backtrack(_)) => {
+                        input.reset(start);
+                        return Ok(acc);
+                    }
+                    Err(e) => return Err(e),
+                    Ok(o) => {
+                        acc.accumulate(o);
                     }
                 }
             }
         }
-    })
+    }
 }
 
 /// [`Accumulate`] the output of a parser, interleaved with `sep`
@@ -387,6 +700,7 @@ where
 /// ```
 #[doc(alias = "sep_by1")]
 #[doc(alias = "separated_list1")]
+#[deprecated(since = "0.5.19", note = "Replaced with `combinator::separated`")]
 pub fn separated1<I, O, C, O2, E, P, S>(mut parser: P, mut sep: S) -> impl Parser<I, C, E>
 where
     I: Stream,
@@ -396,47 +710,206 @@ where
     E: ParserError<I>,
 {
     trace("separated1", move |i: &mut I| {
-        let mut res = C::initial(None);
-
-        // Parse the first element
-        match parser.parse_next(i) {
-            Err(e) => return Err(e),
-            Ok(o) => {
-                res.accumulate(o);
-            }
-        }
-
-        loop {
-            let start = i.checkpoint();
-            let len = i.eof_offset();
-            match sep.parse_next(i) {
-                Err(ErrMode::Backtrack(_)) => {
-                    i.reset(start);
-                    return Ok(res);
-                }
-                Err(e) => return Err(e),
-                Ok(_) => {
-                    // infinite loop check: the parser must always consume
-                    if i.eof_offset() == len {
-                        return Err(ErrMode::assert(i, "sep parsers must always consume"));
-                    }
-
-                    match parser.parse_next(i) {
-                        Err(ErrMode::Backtrack(_)) => {
-                            i.reset(start);
-                            return Ok(res);
-                        }
-                        Err(e) => return Err(e),
-                        Ok(o) => {
-                            res.accumulate(o);
-                        }
-                    }
-                }
-            }
-        }
+        separated1_(&mut parser, &mut sep, i)
     })
 }
 
+fn separated1_<I, O, C, O2, E, P, S>(
+    parser: &mut P,
+    separator: &mut S,
+    input: &mut I,
+) -> PResult<C, E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    P: Parser<I, O, E>,
+    S: Parser<I, O2, E>,
+    E: ParserError<I>,
+{
+    let mut acc = C::initial(None);
+
+    // Parse the first element
+    match parser.parse_next(input) {
+        Err(e) => return Err(e),
+        Ok(o) => {
+            acc.accumulate(o);
+        }
+    }
+
+    loop {
+        let start = input.checkpoint();
+        let len = input.eof_offset();
+        match separator.parse_next(input) {
+            Err(ErrMode::Backtrack(_)) => {
+                input.reset(start);
+                return Ok(acc);
+            }
+            Err(e) => return Err(e),
+            Ok(_) => {
+                // infinite loop check
+                if input.eof_offset() == len {
+                    return Err(ErrMode::assert(
+                        input,
+                        "`separated` separator parser must always consume",
+                    ));
+                }
+
+                match parser.parse_next(input) {
+                    Err(ErrMode::Backtrack(_)) => {
+                        input.reset(start);
+                        return Ok(acc);
+                    }
+                    Err(e) => return Err(e),
+                    Ok(o) => {
+                        acc.accumulate(o);
+                    }
+                }
+            }
+        }
+    }
+}
+
+fn separated_n_<I, O, C, O2, E, P, S>(
+    count: usize,
+    parser: &mut P,
+    separator: &mut S,
+    input: &mut I,
+) -> PResult<C, E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    P: Parser<I, O, E>,
+    S: Parser<I, O2, E>,
+    E: ParserError<I>,
+{
+    let mut acc = C::initial(Some(count));
+
+    if count == 0 {
+        return Ok(acc);
+    }
+
+    match parser.parse_next(input) {
+        Err(e) => {
+            return Err(e.append(input, ErrorKind::Many));
+        }
+        Ok(o) => {
+            acc.accumulate(o);
+        }
+    }
+
+    for _ in 1..count {
+        let len = input.eof_offset();
+        match separator.parse_next(input) {
+            Err(e) => {
+                return Err(e.append(input, ErrorKind::Many));
+            }
+            Ok(_) => {
+                // infinite loop check
+                if input.eof_offset() == len {
+                    return Err(ErrMode::assert(
+                        input,
+                        "`separated` separator parser must always consume",
+                    ));
+                }
+
+                match parser.parse_next(input) {
+                    Err(e) => {
+                        return Err(e.append(input, ErrorKind::Many));
+                    }
+                    Ok(o) => {
+                        acc.accumulate(o);
+                    }
+                }
+            }
+        }
+    }
+
+    Ok(acc)
+}
+
+fn separated_m_n_<I, O, C, O2, E, P, S>(
+    min: usize,
+    max: usize,
+    parser: &mut P,
+    separator: &mut S,
+    input: &mut I,
+) -> PResult<C, E>
+where
+    I: Stream,
+    C: Accumulate<O>,
+    P: Parser<I, O, E>,
+    S: Parser<I, O2, E>,
+    E: ParserError<I>,
+{
+    if min > max {
+        return Err(ErrMode::Cut(E::from_error_kind(input, ErrorKind::Many)));
+    }
+
+    let mut acc = C::initial(Some(min));
+
+    let start = input.checkpoint();
+    match parser.parse_next(input) {
+        Err(ErrMode::Backtrack(e)) => {
+            if min == 0 {
+                input.reset(start);
+                return Ok(acc);
+            } else {
+                return Err(ErrMode::Backtrack(e.append(input, ErrorKind::Many)));
+            }
+        }
+        Err(e) => return Err(e),
+        Ok(o) => {
+            acc.accumulate(o);
+        }
+    }
+
+    for index in 1..max {
+        let start = input.checkpoint();
+        let len = input.eof_offset();
+        match separator.parse_next(input) {
+            Err(ErrMode::Backtrack(e)) => {
+                if index < min {
+                    return Err(ErrMode::Backtrack(e.append(input, ErrorKind::Many)));
+                } else {
+                    input.reset(start);
+                    return Ok(acc);
+                }
+            }
+            Err(e) => {
+                return Err(e);
+            }
+            Ok(_) => {
+                // infinite loop check
+                if input.eof_offset() == len {
+                    return Err(ErrMode::assert(
+                        input,
+                        "`separated` separator parser must always consume",
+                    ));
+                }
+
+                match parser.parse_next(input) {
+                    Err(ErrMode::Backtrack(e)) => {
+                        if index < min {
+                            return Err(ErrMode::Backtrack(e.append(input, ErrorKind::Many)));
+                        } else {
+                            input.reset(start);
+                            return Ok(acc);
+                        }
+                    }
+                    Err(e) => {
+                        return Err(e);
+                    }
+                    Ok(o) => {
+                        acc.accumulate(o);
+                    }
+                }
+            }
+        }
+    }
+
+    Ok(acc)
+}
+
 /// Alternates between two parsers, merging the results (left associative)
 ///
 /// This stops when either parser returns [`ErrMode::Backtrack`]. To instead chain an error up, see
@@ -461,14 +934,14 @@ where
 pub fn separated_foldl1<I, O, O2, E, P, S, Op>(
     mut parser: P,
     mut sep: S,
-    op: Op,
+    mut op: Op,
 ) -> impl Parser<I, O, E>
 where
     I: Stream,
     P: Parser<I, O, E>,
     S: Parser<I, O2, E>,
     E: ParserError<I>,
-    Op: Fn(O, O2, O) -> O,
+    Op: FnMut(O, O2, O) -> O,
 {
     trace("separated_foldl1", move |i: &mut I| {
         let mut ol = parser.parse_next(i)?;
@@ -530,14 +1003,14 @@ where
 pub fn separated_foldr1<I, O, O2, E, P, S, Op>(
     mut parser: P,
     mut sep: S,
-    op: Op,
+    mut op: Op,
 ) -> impl Parser<I, O, E>
 where
     I: Stream,
     P: Parser<I, O, E>,
     S: Parser<I, O2, E>,
     E: ParserError<I>,
-    Op: Fn(O, O2, O) -> O,
+    Op: FnMut(O, O2, O) -> O,
 {
     trace("separated_foldr1", move |i: &mut I| {
         let ol = parser.parse_next(i)?;
@@ -556,73 +1029,6 @@ where
     })
 }
 
-fn repeat_m_n_<I, O, C, E, F>(min: usize, max: usize, parse: &mut F, input: &mut I) -> PResult<C, E>
-where
-    I: Stream,
-    C: Accumulate<O>,
-    F: Parser<I, O, E>,
-    E: ParserError<I>,
-{
-    if min > max {
-        return Err(ErrMode::Cut(E::from_error_kind(input, ErrorKind::Many)));
-    }
-
-    let mut res = C::initial(Some(min));
-    for count in 0..max {
-        let start = input.checkpoint();
-        let len = input.eof_offset();
-        match parse.parse_next(input) {
-            Ok(value) => {
-                // infinite loop check: the parser must always consume
-                if input.eof_offset() == len {
-                    return Err(ErrMode::assert(
-                        input,
-                        "`repeat` parsers must always consume",
-                    ));
-                }
-
-                res.accumulate(value);
-            }
-            Err(ErrMode::Backtrack(e)) => {
-                if count < min {
-                    return Err(ErrMode::Backtrack(e.append(input, ErrorKind::Many)));
-                } else {
-                    input.reset(start);
-                    return Ok(res);
-                }
-            }
-            Err(e) => {
-                return Err(e);
-            }
-        }
-    }
-
-    Ok(res)
-}
-
-fn repeat_n_<I, O, C, E, F>(count: usize, f: &mut F, i: &mut I) -> PResult<C, E>
-where
-    I: Stream,
-    C: Accumulate<O>,
-    F: Parser<I, O, E>,
-    E: ParserError<I>,
-{
-    let mut res = C::initial(Some(count));
-
-    for _ in 0..count {
-        match f.parse_next(i) {
-            Ok(o) => {
-                res.accumulate(o);
-            }
-            Err(e) => {
-                return Err(e.append(i, ErrorKind::Many));
-            }
-        }
-    }
-
-    Ok(res)
-}
-
 /// Repeats the embedded parser, filling the given slice with results.
 ///
 /// This parser fails if the input runs out before the given slice is full.
